import pytest

import b3tojson.fetcher as fetcher

import requests


def test_get_b3_data_no_success(mocker):
    mock_get = mocker.patch.object(requests, 'get')
    mock_get.status_code = 300

    with pytest.raises(Exception):
        fetcher.get_b3_data()

    mock_get.assert_called_once_with(fetcher.B3_NEGOTIABLE_URL, stream=True)



from b3tojson import processor, fetcher

import argparse


def process_args():
    parser = argparse.ArgumentParser(description='B3 to JSON format converter')
    parser.add_argument('--fetch', dest='should_fetch', action='store_true',
                        help='Specifies if data file should be fetched from server')
    parser.add_argument('--b3_file', dest='b3_file', action='store',
                        help='Local B3 data file path')
    parser.add_argument('--json_file', dest='json_file', action='store',
                        help='Target JSON file path', default=fetcher.JSON_FILE)

    args = parser.parse_args()
    if args.should_fetch and args.b3_file is not None:
        print("Ambiguous flags --fetch and --b3_file. Quitting")

    return(args)


def main():
    args = process_args()

    if args.should_fetch:
        b3_file = fetcher.get_b3_data()
    else:
        b3_file = args.b3_file

    obj = processor.FileHandle(b3_file, args.json_file)
    obj.analyze_file()
    obj.save_json()



import requests
import zipfile
import codecs

from os import path, remove

B3_NEGOTIABLE_URL = "http://bvmf.bmfbovespa.com.br/suplemento/ExecutaAcaoDownload.asp?arquivo=Titulos_Negociaveis.zip&server=L"
B3_ENCODING = "iso-8859-1"
FILES_DIR = "files"
TEMP_ZIP = path.join(FILES_DIR, "temp.zip")
JSON_FILE = path.join(FILES_DIR, "stocks_data.json")


def get_b3_data():
    response = requests.get(B3_NEGOTIABLE_URL, stream=True)

    if response.status_code != 200:
        print(f"Could not get resource from server! Status code got: {response.status_code}")
        raise Exception

    with open(TEMP_ZIP, "wb") as fd:
        for chunk in response.iter_content(chunk_size=512):
            fd.write(chunk)

    b3_name = None
    with zipfile.ZipFile(TEMP_ZIP, 'r') as zip_obj:
        zip_obj.extractall(FILES_DIR)

        b3_name = path.join(FILES_DIR, zip_obj.namelist().pop())
        _process_file(b3_name)

    remove(TEMP_ZIP)
    return b3_name


def _process_file(b3_name):
    """
    Process downloaded file in order to fix encoding.

    Args:
        b3_name(str): Path to the original file
    """
    with codecs.open(b3_name, 'r', B3_ENCODING) as source_file:
        contents = source_file.read()

    with codecs.open(b3_name, 'w', 'utf-8') as destination_file:
        destination_file.write(contents)



from b3tojson.data import Company, Stock

import jsonpickle


class FileHandle():
    companies: {Company}

    def __init__(self, in_file, out_file):
        self.in_file = in_file
        self.out_file = out_file
        self.companies = {}

    def _process_raw_data(self, lines):
        lines = [line.strip() for line in lines]
        for line in lines:
            # HEADER -- just skipping it
            if line.startswith('00'):
                continue
            # COMPANY DATA
            elif line.startswith('01'):
                code = line[2:6]
                self.companies[code] = Company(code, line[6:66], line[66:78])
            # STOCK DATA
            elif line.startswith('02'):
                code = line[14:18]
                bdi = line[18:21]
                # 002: STD STOCK // 012: FII
                if bdi == '002' or bdi == '012':
                    self.companies[code].add_stock(Stock(line[2:14], bdi,
                                                         line[21:81],
                                                         line[133:143]))

    def _to_json(self):
        jsonpickle.set_preferred_backend('json')
        jsonpickle.set_encoder_options('json', ensure_ascii=False)
        return jsonpickle.encode(self.companies, unpicklable=False)

    def analyze_file(self):
        try:
            with open(self.in_file, "r", encoding="utf-8") as fd:
                lines = fd.readlines()
        except FileNotFoundError:
            print(f"Could not open file {self.in_file}")
        # TODO: Handle file enconding by iteself
        except UnicodeDecodeError:
            print("Please, ensure the file to be encoded as UTF-8")
        except Exception:
            raise
        else:
            self._process_raw_data(lines)

    def save_json(self):
        encoded = self._to_json()

        with open(self.out_file, 'w', encoding="utf-8") as fd:
            fd.write(encoded)



from dataclasses import dataclass, field

@dataclass
class Company():
    code: str
    social_name: str
    short_name: str
    stocks: list = field(default_factory=list)

    def __post_init__(self):
        self.social_name = self.social_name.strip()

    def __str__(self):
        return(f"{self.code} : {self.social_name} : {self.short_name} + {self.stocks}")

    def add_stock(self, stock):
        self.stocks.append(stock)
        

@dataclass
class Stock():
    ticker: str
    bdi: str
    desc_bdi: str
    spec: str

    def __post_init__(self):
        for var in vars(self):
            setattr(self, var, getattr(self, var).strip())



import os
import sys

import setuptools
import versioneer

sys.path.insert(0, os.path.join(os.path.dirname(__file__), 'src'))  # noqa

setuptools.setup(
    version=versioneer.get_version(),
    cmdclass=versioneer.get_cmdclass()
)
